uniform float windWaveCounter;
uniform float waterWaveCounter;
uniform float windSpeed;
uniform vec3 playerpos;
uniform float globalWarpIntensity;

uniform float glitchStrength = 0;
uniform float windWaveIntensity = 1;


// Re-oriented 8-point BCC noise by K.jpg

vec4 permute(vec4 t) {
	return t * (t * 34 + 1);
}

vec3 grad(float hash) {
	
	// Random vertex of a cube, +/- 1 each
	vec3 cube = mod(floor(hash / vec3(1, 2, 4)), 2) * 2 - 1;
	
	// Random edge of the three edges that vertex connects to
	// Also a cuboctahedral vertex
	// And corresponds to the face of its dual, the rhombic dodecahedron
	vec3 cuboct = cube;
	cuboct[int(hash / 16)] = 0;
	
	// In a funky way, pick one of the four points on the rhombic face
	float type = mod(floor(hash / 8), 2);
	vec3 rhomb = (1 - type) * cube + type * (cuboct + cross(cube, cuboct));
	
	// Expand it by the right amount
	vec3 grad = cuboct * 1.22474487139 + rhomb;
	
	// To make all gradients the same length, we only need to shorten the
    // second type of vector. We also put in the whole noise scale constant.
	grad *= (1.0 - 0.042942436724648037 * type) * 3.5946317686139184;
	
	return grad;
}

float bccnoisePart(vec3 X) {
	vec3 b = floor(X);
	vec4 i4 = vec4(X - b, 2.5);
	
	vec3 v1 = b + floor(dot(i4, vec4(.25)));
	vec3 v2 = b + vec3(1, 0, 0) + vec3(-1, 1, 1) * floor(dot(i4, vec4(-.25, .25, .25, .35)));
	vec3 v3 = b + vec3(0, 1, 0) + vec3(1, -1, 1) * floor(dot(i4, vec4(.25, -.25, .25, .35)));
	vec3 v4 = b + vec3(0, 0, 1) + vec3(1, 1, -1) * floor(dot(i4, vec4(.25, .25, -.25, .35)));
	
	vec4 hashes = permute(mod(vec4(v1.x, v2.x, v3.x, v4.x), 289));
	hashes = permute(mod(hashes + vec4(v1.y, v2.y, v3.y, v4.y), 289));
	hashes = mod(permute(mod(hashes + vec4(v1.z, v2.z, v3.z, v4.z), 289)), 48);
	
	vec3 d1 = v1 - X; vec3 d2 = v2 - X; vec3 d3 = v3 - X; vec3 d4 = v4 - X;
	vec4 a = max(0.75 - vec4(dot(d1, d1), dot(d2, d2), dot(d3, d3), dot(d4, d4)), 0.0);
	a *= a;
	
	return dot(a * a, vec4(dot(d1, grad(hashes.x)), dot(d2, grad(hashes.y)), dot(d3, grad(hashes.z)),  dot(d4, grad(hashes.w))));
}


float bccnoise(vec3 X) {
	// This noise has a slightly higher internal wavelength than Old Perlin,
	// so rescale to make it a more direct replacement
	// TODO this would go better in files that use it
	X *= 0.8660254037844386;
	
	// Rotate, but preserve shape. Hides grid better in cardinal slices.
	// Good for texturing 3D objects with lots of flat parts
	//X = dot(X, vec3(2f/3f)) - X;
	
	// Prioritizes visual isotropy in X and Y, so that Z can be time
	// TODO this can be coded more compactly than a matrix
	X = mat3(
		0.788675134594813, -0.211324865405187, -0.577350269189626,
		-0.211324865405187, 0.788675134594813, -0.577350269189626,
		0.577350269189626, 0.577350269189626, 0.577350269189626) * X;
	
	
	return bccnoisePart(X) + bccnoisePart(X + 144.5);
}

vec4 applyLiquidWarping(bool windAffected, vec4 worldPos) {
	#if WAVINGSTUFF == 1
	vec3 noisepos = vec3((worldPos.x + playerpos.x) / 3, (worldPos.z + playerpos.z) / 3, waterWaveCounter / 8 + (windAffected ? windWaveCounter / 6 : 0));
	worldPos.y += bccnoise(noisepos) / 10;
	#endif
	
	return worldPos;
}

vec4 applyVertexWarping(int renderFlags, vec4 worldPos) {

	#if WAVINGSTUFF == 1

	// bit 12 is the water waving on/off flag
	if ((renderFlags & 0x1000) > 0) {
		vec3 noisepos = vec3((worldPos.x + playerpos.x) / 3, (worldPos.z + playerpos.z) / 3, waterWaveCounter / 8 + windWaveCounter / 6);
		worldPos.y += bccnoise(noisepos) / 10;
	} else {
		// Coopers reed water layer has bit 11 and 12 set. Lets just ignore bit 11 in this case
		
		// bit 11 is the grass swaying on/off flag
		if ((renderFlags & 0x800) > 0 || (renderFlags & 0x8000000) > 0) {
		
			bool isLeaves = ((renderFlags & 0x8000000) > 0);
			
			float div = 30;            
			float freq = (0.4 + windSpeed/10);
			float strength = (0.5 + (isLeaves ? 0 : 0.5)) * windWaveIntensity * (1 + windSpeed);

			float x = freq*(windWaveCounter) + worldPos.x + playerpos.x;
			float y = freq*(windWaveCounter) + worldPos.y + playerpos.y;
			float z = freq*(-windWaveCounter) + worldPos.z  + playerpos.z;
			
			float bendNoise = bccnoise(vec3((worldPos.x+playerpos.x)/10, (worldPos.z+playerpos.z)/10, windWaveCounter/4));
			
			uint flagsbla = uint(renderFlags);
			int groundOffset = int((flagsbla >> 28)) & 0x7;
			
			float heightBend = (isLeaves ? max(groundOffset, 0) / 10.0 : 0.7);
			
			float bend = windSpeed * (1 + bendNoise) * heightBend * windWaveIntensity;
			bend = min(4, bend);
			
			if ((renderFlags & 0x4000) > 0) {
				bend /= 3;
			}

			// Curve visualized: http://fooplot.com/#W3sidHlwZSI6MCwiZXEiOiIyKnNpbih4LzgpK3Npbih4LzIpK3NpbigwLjUrMip4KStzaW4oMSszKngpIiwiY29sb3IiOiIjMDAwMDAwIn0seyJ0eXBlIjoxMDAwLCJ3aW5kb3ciOlsiLTI0Ljc5NTUzMjIyNjU2MjQ4NiIsIjI0Ljc5NTUzMjIyNjU2MjQ4NiIsIi0xNS4yNTg3ODkwNjI0OTk5OTEiLCIxNS4yNTg3ODkwNjI0OTk5OTEiXX1d
			
			worldPos.x += bend + strength * (sin(x*10)/120 + (2 * sin(x/2) + sin(x + y) + sin(0.5 + 4*x + 2*y) + sin(1 + 6*x + 3*y)/3) / div);
			worldPos.y += strength * (sin(5*y)/15 + cos(10*x) / 10 + sin(3*z)/2 + cos(x*2)/2.2)/div;
			worldPos.z += strength * ((2 * sin(z/4) + sin(z + 3 * y) + sin(0.5 + 4*z + 2*y) + sin(1 + 6*z + y)/3) / div);
		}
	}
	
	float str = max(0, glitchStrength - 0.1);
	if (str > 0) {
		str *= min(10, (length(worldPos)-1) * glitchStrength * glitchStrength * glitchStrength);
		worldPos.x += str * bccnoise(vec3((worldPos.x+playerpos.x)/10, (worldPos.z+playerpos.z)/10, windWaveCounter/6)) / 5;
		worldPos.z += str * bccnoise(vec3((worldPos.x+playerpos.x)/10, (worldPos.z+playerpos.z)/10, windWaveCounter/3.5)) / 5;
		worldPos.y += str * bccnoise(vec3((worldPos.x+playerpos.x)/10, (worldPos.z+playerpos.z)/10, windWaveCounter/10)) / 5;
	}
	
	if (globalWarpIntensity > 0) {
		float x = max(0, (mod(20*windWaveCounter, 30)) + (worldPos.x + playerpos.x) / 5.0 + (worldPos.y + playerpos.y) / 8.0 - 40);
		worldPos.x += (sin(x / 2) + sin(0.5 + 2*x) + sin(1 + 3*x)/3) / 30.0 * globalWarpIntensity;
		
		worldPos.z += (cos(x / 3) + cos(0.2 + 2.2*x) + cos(1 + 4*x)/3) / 30.0 * globalWarpIntensity;
	}
	
	#endif
		
	return worldPos;
}




