#version 330 core

uniform sampler2D terrainTex;
uniform float alphaTest = 0.01;
uniform vec2 blockTextureSize;

in vec4 rgba; // Without biome tint
in vec4 rgba2; // With biome tint
in vec4 rgbaFog;
in float fogAmount;
in vec2 uv;
in vec2 uv2;
in float glowLevel;

in float normalShadeIntensity;
flat in int renderFlags;
flat in vec3 normal;


layout(location = 0) out vec4 outColor;
layout(location = 1) out vec4 outGlow;

#include fogandlight.fsh

void main() 
{
	vec4 brownSoilColor = texture(terrainTex, uv) * rgba;
	vec4 grassColor;
	
	if (rgba.a < 0.01) {
		// Bottom
		outColor = applyFog(brownSoilColor, fogAmount);
	} else {
		if (rgba2.a > 0.01) {
			// Top
			grassColor = texture(terrainTex, uv2) * rgba2;
		} else {
			// Side + Overlay
			grassColor = texture(terrainTex, uv2 + vec2(blockTextureSize.x, 0)) * vec4(rgba2.rgb, 1);
		}
		
		outColor = brownSoilColor * (1 - grassColor.a) + grassColor * grassColor.a;
	}
	

	outColor = applyFogAndShadowWithNormal(outColor, fogAmount, normal, normalShadeIntensity, 0.5);

	outColor.a = rgbaFog.a;

	// When looking through tinted glass you can clearly see the edges where we fade to sky color
	// Using this discard seems to completely fix that
	if (outColor.a < 0.005) discard;


	float aTest = outColor.a;
	aTest += max(0, 1 - rgba.a) * min(1, outColor.a * 10);
	if (aTest < alphaTest) discard;
	
    outGlow = vec4(glowLevel, 0, 0, outColor.a);
}