#version 330 core
#extension GL_ARB_explicit_attrib_location: enable

in vec2 uv;
in vec4 color;
in vec4 rgbaFog;
in float fogAmount;
in float glowLevel;

layout(location = 0) out vec4 outColor;
layout(location = 1) out vec4 outGlow;

uniform sampler2D entityTex;
uniform float alphaTest = 0.001;
uniform float windWaveCounter;
uniform float glitchEffectStrength;

flat in int renderFlags;
flat in vec3 normal;

#include fogandlight.fsh
#include noise3d.ash

void main () {
	vec4 texColor = texture(entityTex, uv) * color;
	
	outColor = applyFogAndShadowWithNormal(texColor, fogAmount, normal, 1, 0.35);

	/*outColor.r = normal.x;
	outColor.g = normal.y;
	outColor.b = normal.z;*/
	
	if (glitchEffectStrength > 0) {
		float g = cnoise(vec3(gl_FragCoord.y / 2.0, gl_FragCoord.x / 2.0, windWaveCounter*30));
		outColor.a *= mix(1, clamp(0.7 + g / 2, 0, 1), glitchEffectStrength);
		
		float b = cnoise(vec3(0, 0, windWaveCounter*30));
		outColor.a *= mix(1, clamp(b * 10 + 4, 0, 1), glitchEffectStrength);
	}

	if (outColor.a < alphaTest) discard;
	
	outGlow = vec4(glowLevel, 0, 0, color.a);
}